# -*- coding: utf-8 -*-

import os
import re
from django.db import models
from django.conf import settings
from django.forms import ModelForm
from django_extensions.db.fields import AutoSlugField
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify
from django.db.models.signals import  post_save
from django.dispatch.dispatcher import receiver


facebook_regexp = re.compile(r"^https?://[^/]*(facebook\.com|fb\.com|fb\.me)(/.*|/?)")
twitter_regexp = re.compile(r"^https?://[^/]*(t\.co|twitter\.com)(/.*|/?)")
http_regexp = re.compile(r"^(ht|f)tps?://.*")


# Create your models here.
class Election(models.Model):
    name = models.CharField(max_length=255, verbose_name=_(u"NOMBRE DE LA ELECCIÓN:"))
    slug = models.CharField(max_length=255, verbose_name=_("Con este link podras acceder a la eleccion:"))
    owner = models.ForeignKey('auth.User')
    description = models.TextField(_(u"DESCRIPCIÓN DE LA ELECCIÓN:"), max_length=10000)
    information_source = models.TextField(_(u"DE DONDE OBTUVISTE LA INFORMACIÓN:"), max_length=10000, blank = True)
    logo = models.ImageField(upload_to = 'logos/', blank = True, verbose_name=_(u"por último escoge una imagen que la represente:"))
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)
    date = models.CharField(max_length=255, verbose_name=_(u"fecha en que ocurrirá:"), blank=True)
    published = models.BooleanField(default=False)
    custom_style = models.TextField(blank=True)
    highlighted = models.BooleanField(default=False)
    use_default_media_naranja_option = models.BooleanField(default=True) #Default option "Ninguna de las anteriores" in media naranja
    should_display_empty_personal_data = models.BooleanField(default=True, verbose_name=_(u"debe mostrar los datos personales vacíos:"))
    

    class Meta:
        unique_together = ('owner', 'slug')

    def __unicode__(self):
        return u"%s" % self.name
    
    def set_slug(self):
        if not self.slug and self.name and self.owner:
            existing_elections = Election.objects.all().filter(owner= self.owner, name=self.name)
            slug = slugify(self.name)
            not_unique_slug = Election.objects.all().filter(owner= self.owner).filter(slug=slug).exists()
            previous_elections = 1
            temporary_slug = slug
            while not_unique_slug:
                temporary_slug = slug
                previous_elections += 1
                temporary_slug += str(previous_elections)
                not_unique_slug = Election.objects.all().filter(owner= self.owner).filter(slug=temporary_slug).exists()
            slug = temporary_slug
            self.slug = slug

    def __init__(self, *args, **kwargs):
        super(Election, self).__init__(*args, **kwargs)
        self.set_slug()
        
        
        

    @models.permalink
    def get_absolute_url(self):
        return ('election_detail', None, {'username': self.owner.username, 'slug': self.slug})

class Candidate(models.Model):
    name = models.CharField(max_length=255, verbose_name=_(u"Nombre:"))
    slug = models.CharField(max_length=255, blank=True)
    photo = models.ImageField(upload_to = 'photos/', blank = True)

    election = models.ForeignKey('Election')
    answers = models.ManyToManyField('Answer', blank=True)

    personal_data = models.ManyToManyField('PersonalData', through='PersonalDataCandidate')
    background = models.ManyToManyField('Background', through='BackgroundCandidate')
    has_answered = models.BooleanField(default=True, verbose_name=_("Has answered?"))
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)


    def save(self, *args, **kwargs):
        if self.pk is None:
            self.slug = slug = slugify(self.name)
            counter = 1
            while True:
                try:
                    Candidate.objects.get(slug=self.slug, election=self.election)
                    self.slug = slug + str(counter)
                    counter += 1
                except self.DoesNotExist:
                    break

        super(Candidate, self).save(*args, **kwargs)

    class Meta:
        unique_together = (('slug', 'election'), ('name', 'election'))

    def associate_answer(self, answer):
        old_answers = self.answers.filter(question=answer.question).all()
        new_answers = []
        for ans in self.answers.all():
            if ans not in old_answers:
                new_answers.append(ans)
        new_answers.append(answer)
        self.answers = new_answers
        self.save()

    def get_number_of_questions_by_category(self):
        number_of_questions = [] # Number of questions per category
        categories = Category.objects.filter(election=self.election)
        for category in categories:
            questions = Question.objects.filter(category=category)
            number_of_questions.append(len(questions))
        return number_of_questions

    def get_importances_by_category(self, importances):
        number_of_questions = self.get_number_of_questions_by_category()
        importances_by_category = []
        index = 0
        for num in number_of_questions:
            category_importance = 0.0
            for i in range(num):
                category_importance += importances[index]
                index += 1
            importances_by_category.append(category_importance)
        return importances_by_category

    def get_sum_importances_by_category(self, answers, importances):
        categories = Category.objects.filter(election=self.election)
        sum_by_category = [0]*len(categories)
        candidate_answers = self.answers.all() # Candidate answers
        index = 0
        # example answers = [[], [], [<Answer: Si>], [<Answer: Si>]]
        for x in reversed(answers):
            if len(x) != 0:
                if x[0] in candidate_answers:
                    factor = 1
                    value = factor * importances[len(importances)-index-1]
                    pos = list(categories.all()).index(x[0].question.category)
                    sum_by_category[pos] += value
                else:
                    factor = 0
            else:
                factor = 0
            index += 1
        return sum_by_category

    def get_score(self, answers, importances):
        sum_by_category = self.get_sum_importances_by_category(answers, importances)
        importances_by_category = self.get_importances_by_category(importances)
        scores_by_category = []
        for i in range(len(sum_by_category)):
            if importances_by_category[i] != 0:
                scores_by_category.append(sum_by_category[i]*100.0/importances_by_category[i])
            else:
                scores_by_category.append(0)
        if len(importances) > 0:
            try:
                return ((sum(sum_by_category)*100.0/sum(importances)),scores_by_category)
            except :
                return (0,scores_by_category)
        else:
            return (0,scores_by_category)

    def add_background(self, background, value):
        bcs = BackgroundCandidate.objects.filter(background=background, candidate=self)
        if len(bcs) > 0:
            for bc in bcs:
                bc.delete()
        background_candidate = BackgroundCandidate(background=background, candidate=self, value=value)
        background_candidate.save()

    def add_personal_data(self, personal_data, value):
        pdc = PersonalDataCandidate.objects.filter(personal_data=personal_data, candidate=self)
        if len(pdc) > 0:
            for pd in pdc:
                pd.delete()
        personal_data = PersonalDataCandidate(personal_data=personal_data, candidate=self, value=value)
        personal_data.save()

    @property
    def get_background(self):
        backgrounds = {}
        category_counter = 0
        for backgroundcategory in self.election.backgroundcategory_set.all():
            category_counter = category_counter + 1
            backgrounds[category_counter] = {'name':backgroundcategory.name, 'backgrounds':None}
            temporary_backgrounds = {}
            backgrounds_counter = 0
            for background in backgroundcategory.background_set.all():
                backgrounds_counter = backgrounds_counter + 1
                what_the_candidate_answered = self.get_answer_for_background(background)
                temporary_backgrounds[backgrounds_counter] = {
                                                    'name':background.name, 
                                                    'value':what_the_candidate_answered
                                                    }
            backgrounds[category_counter]['backgrounds'] = temporary_backgrounds
        return backgrounds
        
        
    def get_answer_for_background(self, background):
        try:
            return self.backgroundcandidate_set.get(background=background).value
        except:
            return None

    @property
    def get_personal_data(self):
        pd_dict = {}
        for pd in self.election.personaldata_set.all():
            try:
                pd_dict[pd.label] = self.personaldatacandidate_set.get(personal_data = pd).value
            except :
                pd_dict[pd.label] = None
        return pd_dict

    def get_questions_by_category(self, category):
        return category.question_set.all()

    def get_answer_by_question(self, question):
        candidate_answers = self.answers
        for answer in candidate_answers.all():
            if answer.question == question:
                return answer
        return "no answer"

    def get_all_answers_by_category(self, category):
        all_answers = []
        all_questions = self.get_questions_by_category(category)
        for question in all_questions:
            candidate_answer = self.get_answer_by_question(question)
            all_answers.append((question, candidate_answer))
        return all_answers

    def get_answers_two_candidates(self, candidate, category):
        all_answers = []
        all_questions = self.get_questions_by_category(category)
        for question in all_questions:
            first_candidate_answer = self.get_answer_by_question(question)
            second_candidate_answer = candidate.get_answer_by_question(question)
            all_answers.append((question,first_candidate_answer,second_candidate_answer))
        return all_answers

    def __unicode__(self):
        return self.name


class PersonalData(models.Model):
    label = models.CharField(_('Nuevo dato personal'),max_length=255)
    election = models.ForeignKey('Election')

    def __unicode__(self):
        return u'%s (%s)' % (self.label, self.election)


class PersonalDataCandidate(models.Model):
    candidate = models.ForeignKey(Candidate)
    personal_data = models.ForeignKey(PersonalData)
    value = models.TextField(max_length=2000)

    def __unicode__(self):
        return self.candidate.name + " - " + self.personal_data.label


class BackgroundCategory(models.Model):
    name = models.CharField(_(u"Nueva categoría de antecedentes"), max_length=255)
    election = models.ForeignKey('Election')

    def __unicode__(self):
        return u'%s (%s)' % (self.name, self.election)


class Background(models.Model):
    name = models.CharField(max_length=255)
    category = models.ForeignKey('BackgroundCategory')
    def __unicode__(self):
        return u'%s: %s (%s)' % (self.category, self.name, self.category.election)

class BackgroundCandidate(models.Model):
    candidate = models.ForeignKey('Candidate')
    background = models.ForeignKey('Background')
    value = models.CharField(max_length=2000)

    def __unicode__(self):
        return u'%s: %s' % (self.candidate, self.value)



class Link(models.Model):
    name = models.CharField(max_length=255)
    url = models.CharField(max_length=255)
    candidate = models.ForeignKey('Candidate')

    @property
    def css_class(self):
        if facebook_regexp.match(self.url):
            return "facebook"
        elif twitter_regexp.match(self.url):
            return "twitter"

    def __unicode__(self):
        return u"%s (%s)" % (self.name, self.url)

    @property
    def http_prefix(self):
        if http_regexp.match(self.url):
            return self.url
        else:
            return "http://"+self.url


class Category(models.Model):
    name = models.CharField(max_length=255)
    election = models.ForeignKey('Election')
    slug = models.CharField(max_length=255, blank=True)
    order = models.PositiveIntegerField(default=1)

    def save(self, *args, **kwargs):
        if self.pk is None:
            self.slug = slug = slugify(self.name)
            counter = 1
            while True:
                try:
                    Category.objects.get(slug=self.slug, election=self.election)
                    self.slug = slug + str(counter)
                    counter += 1
                except self.DoesNotExist:
                    break

        super(Category, self).save(*args, **kwargs)

    class Meta:
        unique_together = (('election', 'slug'), ('name', 'election'))
        verbose_name_plural = 'Categories'
        ordering = ['order']

    def __unicode__(self):
        return u"%s" % self.name


class Question(models.Model):
    question = models.CharField(max_length=255, verbose_name=_("PREGUNTA:"))
    category = models.ForeignKey('Category', verbose_name=_("CATEGORIA:"))

    def __unicode__(self):
        return u"%s" % self.question


class Answer(models.Model):
    caption = models.CharField(max_length=255)
    question = models.ForeignKey('Question')

    def __unicode__(self):
        return u"%s" % self.caption

class Visitor(models.Model):
    election = models.ForeignKey('Election')
    election_url = models.CharField(max_length=255)
    datestamp = models.DateTimeField(auto_now=True)
    def __unicode__(self):
        return str(self.datestamp) + ' - ' + self.election_url 
        
class VisitorAnswer(models.Model):
    """docstring for VisitorAnswer"""
    visitor = models.ForeignKey('Visitor')
    answer_text = models.CharField(max_length=255)
    question_text = models.CharField(max_length=255)
    question_category_text = models.CharField(max_length=255)
    answer_importance = models.IntegerField()
    def __init__(self, *args, **kwargs):
        if 'answer' in kwargs:
            kwargs['answer_text'] = kwargs['answer'].caption
            kwargs['question_text'] = kwargs['answer'].question.question
            kwargs['question_category_text'] = kwargs['answer'].question.category.name
            del kwargs['answer']
        super(VisitorAnswer, self).__init__(*args, **kwargs)

class VisitorScore(models.Model):
    visitor = models.ForeignKey('Visitor')
    candidate_name = models.CharField(max_length=255)
    score = models.IntegerField()

class CategoryScore(models.Model):
    visitor_score = models.ForeignKey('VisitorScore')
    category_score = models.IntegerField()
    category_name = models.CharField(max_length=255)       

    



@receiver(post_save, sender=Election)
def create_default_personaldata(sender, instance, created, **kwargs):
    if created:
        for label in settings.DEFAULT_PERSONAL_DATA:
            PersonalData.objects.create(label=label, election=instance)

@receiver(post_save, sender=Election)
def create_default_backgrounds(sender, instance, created, **kwargs):
    if created:
        for name in settings.DEFAULT_BACKGROUND_CATEGORIES:
            category = BackgroundCategory.objects.create(name=name, election=instance)
            for background in settings.DEFAULT_BACKGROUND_CATEGORIES[name]:
                Background.objects.create(name=background, category=category)

@receiver(post_save, sender=Election)
def create_default_questions(sender, instance, created, **kwargs):
    if created:
        for default_category in settings.DEFAULT_QUESTIONS:
            category = Category.objects.create(name=default_category['Category'], election=instance)
            for default_question in default_category['Questions']:
                question = Question.objects.create(question=default_question['question'],category=category)
                for default_answer in default_question['answers']:
                    Answer.objects.create(question=question, caption=default_answer)


class InformationSource(models.Model):
    question = models.ForeignKey(Question)
    candidate = models.ForeignKey(Candidate)
    content = models.TextField()


from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError

from elections.models import Election, BackgroundCategory
from elections.forms import BackgroundCategoryForm


class BackgroundCategoryModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

    def test_create_background_category(self):
        background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='foo')
        self.assertTrue(created)
        self.assertEqual(background_category.name, 'foo')
        self.assertEqual(background_category.election, self.election)


class AsyncDeleteBackgroundCategoryTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.category = BackgroundCategory.objects.create(name="Bar1", election=self.election)

    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        response = self.client.post(reverse('async_delete_background_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})
        self.assertEquals(response.status_code, 200)

    def test_post_without_login(self):
        response = self.client.post(reverse('async_delete_background_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})
        self.assertEquals(response.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_delete_background_category',
                                kwargs={'category_pk': self.category.pk}))
        self.assertEquals(response.status_code, 405)

    def test_post_with_stranger_election(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        category2 = BackgroundCategory.objects.create(name="Bar1", election=election2)

        self.client.login(username='joe', password='doe')
        response = self.client.post(reverse('async_delete_background_category',
                                kwargs={'category_pk': category2.pk}))

        self.assertEquals(response.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.category.pk
        request = self.client.post(reverse('async_delete_background_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(BackgroundCategory.DoesNotExist, BackgroundCategory.objects.get, pk=temp_pk)


class BackgroundCategoryCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

    def test_create_background_category_by_user_without_login(self):
        request = self.client.get(reverse('background_category_create',
                                    kwargs={'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

    def test_create_background_category_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('background_category_create',
                                    kwargs={'election_slug': self.election.slug}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], BackgroundCategoryForm))
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))
    
    def test_renders_step_four_template(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('background_category_create',
                                    kwargs={'election_slug': self.election.slug}))
        
        self.assertTemplateUsed(response, 'elections/wizard/step_four.html')

    def test_post_background_category_create_without_login(self):
        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_background_category_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('background_category_create',
                                    kwargs={'election_slug': 'strager_election_slug'}))
        self.assertEquals(response.status_code, 404)

    def test_post_background_category_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_category_create',
                                        kwargs={'election_slug': 'strager_election_slug'}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_background_category_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = BackgroundCategory.objects.filter(name='Bar')
        self.assertEquals(qs.count(), 1)
        background_category = qs.get()
        self.assertEqual(background_category.name, params['name'])
        self.assertEqual(background_category.election, self.election)

        self.assertRedirects(response, reverse('background_category_create',
                                               kwargs={'election_slug': self.election.slug}))

class AsyncCreateBackgroundCategoryViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.background_category2, created = BackgroundCategory.objects.get_or_create(election=self.election2,
                                                                    name='FooBar')


    def test_get_async_create_background_category_with_login(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_create_background_category',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 405)

    def test_get_async_create_background_category_without_login(self):
        response = self.client.get(reverse('async_create_background_category',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 302)


    def test_post_async_create_background_category_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background_category',
                                    kwargs={'election_pk': self.election.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_post_async_create_background_category_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background_category',
                                    kwargs={'election_pk': self.election2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_async_create_background_category_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background_category',
                                    kwargs={'election_pk': self.election.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)

        background_categories = self.election.backgroundcategory_set.all()
        background_categories_names = [ background_category.name for background_category in background_categories]

        self.assertTrue(params['value'] in background_categories_names)

        background_category = self.election.backgroundcategory_set.get(name=params['value'])
        self.assertEquals(response.content, '{"pk": %d, "name": "%s"}' % (background_category.pk, params['value']))



from django.conf.urls.defaults import patterns, url
from django.views.generic import DetailView, ListView, TemplateView, CreateView
from django.views.generic.simple import direct_to_template
from django.contrib.auth.decorators import login_required
from elections.views.answer_views import AnswerCreateAjaxView

from models import Election
from views.election_views import ElectionUpdateDataView
from views.candidate_views import CandidateUpdatePhotoView
from views import associate_answer_to_candidate, ElectionCreateView,\
                  ElectionUpdateView, ElectionDetailView, CandidateDetailView,\
                  CandidateCreateView, CandidateUpdateView, CategoryCreateView,\
                  CandidateCreateAjaxView, CategoryUpdateView, PersonalDataCreateView,\
                  BackgroundCategoryCreateView, BackgroundCreateView, QuestionCreateView,\
                  AnswerCreateView, personal_data_candidate_create, background_candidate_create,\
                  CandidateDataUpdateView, async_delete_candidate, async_create_background, \
                  async_delete_background, async_delete_background_category, \
                  PrePersonalDataView, AnswerDeleteAjaxView, ElectionLogoUpdateView, \
                  ElectionShareView, ElectionRedirectView, HomeTemplateView, CompareView, \
                  ElectionAboutView, ElectionStyleUpdateView, EmbededTemplateView, \
                  UserElectionsView, TogglePublishView


urlpatterns = patterns('',
    



    #frontend embed
    # Election detail view embeded
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/embeded/?$', ElectionDetailView.as_view(template_name="elections/embeded/election_detail_profiles.html"), name='election_detail_embeded'),
    # Election candidates profiles
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/profiles/embeded/?$', ElectionDetailView.as_view(template_name='elections/embeded/election_detail_profiles.html'), name='election_detail_profiles_embeded'),
    # Media Naranja
    url(r'^(?P<username>[a-zA-Z0-9-]+)/(?P<election_slug>[a-zA-Z0-9-]+)/medianaranja/embeded/?$', 'candidator.elections.views.medianaranja1_embed',name='medianaranja1_embeded'),

    # Election compare view
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/embeded/?$', CompareView.as_view(template_name='elections/embeded/election_compare.html'), name='election_compare_embeded'),

    # Election compare view with both candidates (and considering one category)
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/(?P<second_candidate_slug>[-\w]+)/(?P<category_slug>[-\w]+)/embeded/?$', CompareView.as_view(template_name='elections/embeded/election_compare.html'), name='election_compare_two_candidates_embeded'),

    # Election compare view with both candidates (and NO category)
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/(?P<second_candidate_slug>[-\w]+)/embeded/?$', CompareView.as_view(template_name='elections/embeded/election_compare.html'), name='election_compare_two_candidates_and_no_category_embeded'),

    # Election compare view with 1 candidate
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/embeded?$', CompareView.as_view(template_name='elections/embeded/election_compare.html'), name='election_compare_one_candidate_embeded'),
     # Election description
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/about/embeded?$', ElectionAboutView.as_view(template_name="elections/embeded/about.html"), name='election_about_embeded'),

    #frontend
    # Election candidates profiles





    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/profiles/?$', ElectionDetailView.as_view(template_name='elections/election_detail_profiles.html'), name='election_detail_profiles'),

    # Election compare view
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/?$', CompareView.as_view(template_name='elections/election_compare.html'), name='election_compare'),

    # Election compare view with both candidates (and considering one category)
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/(?P<second_candidate_slug>[-\w]+)/(?P<category_slug>[-\w]+)/?$', CompareView.as_view(template_name='elections/election_compare.html'), name='election_compare_two_candidates'),

    # Election compare view with both candidates (and NO category)
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/(?P<second_candidate_slug>[-\w]+)/?$', CompareView.as_view(template_name='elections/election_compare.html'), name='election_compare_two_candidates_and_no_category'),

    # Election compare view with 1 candidate
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare/(?P<first_candidate_slug>[-\w]+)/?$', CompareView.as_view(template_name='elections/election_compare.html'), name='election_compare_one_candidate'),

    # Asynchronous call for compare view
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/compare-async/(?P<candidate_slug>[-\w]+)/?$', 'candidator.elections.views.election_compare_asynchronous_call', name='election_compare_asynchronous_call'),

    # Election description
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/about/?$', ElectionAboutView.as_view(), name='election_about'),
    # Election detail view
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/$', ElectionDetailView.as_view(template_name='elections/election_detail_profiles.html'), name='election_detail'),

    # Root: login_required (por ahora pues no se ha definido un index)
    url(r'^$', HomeTemplateView.as_view(), name="home"),
    #Election redirects to my elections if I have at least one election or to create
    url(r'^election_redirect/$', login_required(ElectionRedirectView.as_view()), name='election_redirect'),
    # My Elections
    url(r'^my_election_list/$', login_required(TemplateView.as_view(template_name="elections/my_election_list.html")), name='my_election_list'),

    # Associate Candidate
    url(r'^(?P<election_slug>[-\w]+)/(?P<candidate_slug>[-\w]+)/associate_answers/',
            associate_answer_to_candidate,
            name='associate_answer_candidate'),

    # Create Answer Ajax
    url(r'answer_create/(?P<question_pk>\d+).json', AnswerCreateAjaxView.as_view(), name='answer_create_ajax'),
    
    #Delete Answer Ajax
    url(r'answer_delete/(?P<pk>\d+)', AnswerDeleteAjaxView.as_view(), \
        name='answer_delete_ajax'),


    # Create Link Ajax
    url(r'^(?P<candidate_pk>\d+)/create_link', 'candidator.elections.views.async_create_link', name='link_create_ajax'),

    # Estaba al final, cual queda?
    # Create Answer Ajax
    # url(r'^(?P<question_pk>[0-9]+)/answer/ajax_create/?$', answer_ajax_create, name='answer_ajax_create'),

    # Create Category View
    url(r'^(?P<election_slug>[-\w]+)/category/create/?$', CategoryCreateView.as_view(), name='category_create'),

    # Update Category view
    url(r'^(?P<election_slug>[-\w]+)/category/(?P<slug>[-\w]+)/update/?$', CategoryUpdateView.as_view(), name='category_update'),

    # Create Question view
    url(r'^(?P<election_slug>[-\w]+)/question/create/?$', QuestionCreateView.as_view(), name='question_create'),

    # Create Answer view
    url(r'^(?P<question_pk>[0-9]+)/answer/create/?$', AnswerCreateView.as_view(), name='answer_create'),

    # Create election view
    url(r'^election/create/?$', ElectionCreateView.as_view(), name='election_create'),

    # Pre-Create election view
    url(r'^election/pre_create/?$', login_required(TemplateView.as_view(template_name="elections/election_pre_create.html")), name='election_pre_create'),

    # Election update view
    url(r'^election/(?P<slug>[-\w]+)/update/?$', ElectionUpdateView.as_view(), name='election_update'),

    # Election style update view
    url(r'^election/(?P<slug>[-\w]+)/update_style/?$', ElectionStyleUpdateView.as_view(), name='update_custom_style'),

    # Election share view
    url(r'^election/(?P<slug>[-\w]+)/share/?$', ElectionShareView.as_view(template_name='elections/updating/share.html'), name='share_my_election'),

    # Election detail view admin
    url(r'^(?P<username>[-\w]+)/(?P<slug>[-\w]+)/gracias$', ElectionDetailView.as_view(template_name='elections/wizard/thanks_for_using_us.html'), name='election_detail_admin'),

    url(r'^election/(?P<slug>[-\w]+)/questions/?', ElectionUpdateDataView.as_view(), name='election_update_data'),
    
    url(r'^election/(?P<pk>\d+)/update_election_photo', ElectionLogoUpdateView.as_view(), name="update_election_photo"),

    # Toggle publish election
    url(r'^elections/toggle_publish/(?P<pk>[-\d]+)/?$', TogglePublishView.as_view() , name = 'toggle_publish' ),

    # Media Naranja
    url(r'^(?P<username>[a-zA-Z0-9-]+)/(?P<election_slug>[a-zA-Z0-9-]+)/medianaranja$', 'candidator.elections.views.medianaranja1',name='medianaranja1'),
    




    # Update candidate view
    url(r'^(?P<election_slug>[-\w]+)/candidate/(?P<slug>[-\w]+)/update/?$', CandidateUpdateView.as_view(), name='candidate_update'),

    url(r'^election/update/candidate/(?P<pk>\d+)/photo/$', CandidateUpdatePhotoView.as_view(), name='update_candidate_photo'),

    # Create candidate view
    url(r'^(?P<election_slug>[-\w]+)/candidate/create/?$', CandidateCreateView.as_view(template_name='elections/wizard/step_two.html'),name='candidate_create'),

    # Create candidate view not in wizard
    url(r'^(?P<election_slug>[-\w]+)/candidate/create_alone/?$', CandidateCreateView.as_view(template_name='elections/updating/candidates.html'), name='candidate_create_alone'),

    # Create candidate using next button
    url(r'^(?P<election_slug>[-\w]+)/candidate/save_candidate/?$', CandidateCreateAjaxView.as_view(), name='async_create_candidate'),

    # List Candidates in json
    url(r'^(?P<username>[a-zA-Z0-9-]+)/(?P<election_slug>[a-zA-Z0-9-]+)/candidate_list.json/?$', 'candidator.elections.views.candidate_views.get_candidate_list_as_json', name='candidate_list_json'),

    # Delete candidate view in wizzard
    url(r'^candidate/async_delete/?$', 'candidator.elections.views.async_delete_candidate' , name='async_delete_candidate'),

    # Candidate data Update (PersonalData and Background)
    url(r'^(?P<election_slug>[-\w]+)/candidate/(?P<slug>[-\w]+)/data_update/?$', CandidateDataUpdateView.as_view(template_name="elections/updating/answers.html"), name='candidate_data_update'),

    # Multiple candidate data Update (answering questions for several candidates)
    url(r'^(?P<election_slug>[-\w]+)/multiple_candidate_data_update/?$', CandidateDataUpdateView.as_view(template_name="elections/updating/answers.html"), name='multiple_candidate_data_update'),

    # Multiple candidate data Update (answering questions for several candidates) but this is just the first time ritght after the wizard
    url(r'^(?P<election_slug>[-\w]+)/multiple_candidate_data_update_first_time/?$', CandidateDataUpdateView.as_view(template_name="elections/updating/answers.html", first_time=True), name='multiple_candidate_data_update_first_time'),

    # Pre-Create PersonalData
    url(r'^(?P<election_slug>[-\w]+)/pre_personaldata/?$', login_required(PrePersonalDataView.as_view(template_name="elections/wizard/step_two_point_five.html")), name='pre_personaldata'),

    # Create PersonalData
    url(r'^(?P<election_slug>[-\w]+)/personal_data/create/?$', PersonalDataCreateView.as_view(), name='personal_data_create'),

    # Create PersonalData Ajax
    url(r'^(?P<election_pk>[-\w]+)/personal_data/async_create/?$', 'candidator.elections.views.async_create_personal_data', name='async_create_personal_data'),

    # Delete personalData view in wizzard
    url(r'^(?P<personal_data_pk>[-\d]+)/personal_data/async_delete/?$', 'candidator.elections.views.async_delete_personal_data', name='async_delete_personal_data'),

    # Create PersonalDataCandidate
    url(r'^(?P<candidate_pk>[0-9]+)/(?P<personal_data_pk>[0-9]+)/personal_data_associate' , 'candidator.elections.views.personal_data_candidate_create', name='personal_data_candidate_create'),

    # Create BackgroundCategory
    url(r'^(?P<election_slug>[-\w]+)/background_category/create/?$', BackgroundCategoryCreateView.as_view(), name='background_category_create'),

    # Create BackgroundCategory Ajax
    url(r'^(?P<election_pk>[-\w]+)/background_category/async_create/?$', 'candidator.elections.views.async_create_background_category', name='async_create_background_category'),

     # Delete BackgroundCategory view in wizzard
    url(r'^(?P<category_pk>[-\d]+)/background_category/async_delete/?$', 'candidator.elections.views.async_delete_background_category' , name='async_delete_background_category'),

    # Create Background
    url(r'^(?P<background_category_pk>[0-9]+)/background/create/?$', BackgroundCreateView.as_view(), name='background_create'),

    # Create Background Ajax
    url(r'^(?P<background_category_pk>[0-9]+)/background/async_create/?$', 'candidator.elections.views.async_create_background', name='async_create_background'),

    # Delete background view in wizzard
    url(r'^(?P<background_pk>[-\d]+)/background/async_delete/?$', 'candidator.elections.views.async_delete_background' , name='async_delete_background'),

    # Create Category Ajax
    url(r'^(?P<election_pk>[-\w]+)/category/async_create/?$', 'candidator.elections.views.async_create_category', name='async_create_category'),

    # Delete category view in wizzard
    url(r'^(?P<category_pk>[0-9]+)/category/async_delete/?$', 'candidator.elections.views.async_delete_category' , name='async_delete_category'),

    # Create BackgroundCandidate
    url(r'^(?P<candidate_pk>[0-9]+)/(?P<background_pk>[0-9]+)/background_associate' , background_candidate_create, name='background_candidate_create'),

    # Delete question view in wizzard
    url(r'^(?P<question_pk>[0-9]+)/question/async_delete/?$', 'candidator.elections.views.async_delete_question' , name='async_delete_question'),

    # Create question view in wizzard
    url(r'^(?P<category_pk>[0-9]+)/question/async_create/?$', 'candidator.elections.views.async_create_question' , name='async_create_question'),

    # Delete link ajax
    url(r'^(?P<link_pk>[-\d]+)/link/async_delete/?$', 'candidator.elections.views.async_delete_link' , name='async_delete_link'),

    

    # Candidate detail view
    url(r'^(?P<username>[-\w]+)/(?P<election_slug>[-\w]+)/(?P<slug>[-\w]+)$', CandidateDetailView.as_view(), name='candidate_detail'),
    #embeded

    # Candidate detail view
    url(r'^(?P<username>[-\w]+)/(?P<election_slug>[-\w]+)/(?P<slug>[-\w]+)/embeded/?$', CandidateDetailView.as_view(template_name="elections/embeded/candidate_detail.html"), name='candidate_detail_embeded'),





    # Solo para efectos de prueba del embeded
    url(r'^prueba_embeded$', EmbededTemplateView.as_view(), name="prueba_embeded"),

    #Municipales 2012

    url(r'^municipales2012/?$', TemplateView.as_view(template_name="municipales2012.html"), name="municipales2012"),


    # Election detail view
    url(r'^(?P<username>[-\w]+)/?$', UserElectionsView.as_view(), name='user_elections'),


)



# myapp/api.py
from tastypie.authentication import ApiKeyAuthentication
from tastypie.constants import ALL, ALL_WITH_RELATIONS
from tastypie.resources import ModelResource
from elections.models import Election, Candidate, Category, Question, Answer, \
							PersonalData, PersonalDataCandidate, Link, Background, BackgroundCandidate
from tastypie import fields
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

class PersonalDataResource(ModelResource):
	class Meta:
		queryset = PersonalData.objects.all()

class LinkResource(ModelResource):
	class Meta:
		queryset = Link.objects.all()

class BackgroundResource(ModelResource):
	class Meta:
		queryset = Background.objects.all()

class CandidateResource(ModelResource):
	personal_data = fields.ManyToManyField(PersonalDataResource, 'personal_data', null=True, full=True)
	links = fields.ToManyField(LinkResource, 'link_set', full=True)
	background = fields.ManyToManyField(BackgroundResource, 'background', full=True)

	class Meta:
		queryset = Candidate.objects.all()
		resource_name = 'candidate'
		authentication = ApiKeyAuthentication()

	def authorized_read_list(self, object_list, bundle):
		return object_list.filter(election__owner=bundle.request.user)

	def dehydrate(self, bundle):
		candidate = bundle.obj
		categories = bundle.obj.election.category_set.all()
		bundle.data["categories"] = []
		
		for category in categories:
			questions_array = []
			for question in category.question_set.all():
				answers = Answer.objects.filter(question=question).filter(candidate=candidate)
				the_answer = None
				if len(answers)==1:
					the_answer = {
						"id":answers[0].id,
						"caption":answers[0].caption
					}
				questions_array.append({
						"id": question.id,
						"answer": the_answer,
						"question": question.question
					})
			
			category_dict = {
				"id":category.id,
				"name":category.name,
				"questions":questions_array
			}

			bundle.data["categories"].append(category_dict)


		for pdata in bundle.data['personal_data']:
			personal_data_candidate = PersonalDataCandidate.objects.get(candidate=bundle.obj, personal_data=pdata.obj)
			pdata.data['value'] = personal_data_candidate.value
			del pdata.data['resource_uri']
		
		for data in bundle.data['background']:
			background_candidate = BackgroundCandidate.objects.get(candidate=bundle.obj, background=data.obj)
			data.data['value'] = background_candidate.value

		return bundle

class AnswerResource(ModelResource):
	class Meta:
		queryset = Answer.objects.all()
		resource_name = 'answer'

class QuestionResource(ModelResource):
	possible_answers = fields.ToManyField(AnswerResource, 'answer_set', null=True, full=True)
	class Meta:
		queryset= Question.objects.all()
		resource_name = 'question'
		authentication = ApiKeyAuthentication()

class CategoryResource(ModelResource):
	questions = fields.ToManyField(QuestionResource, 'question_set', null=True, full=True)
	class Meta:
		queryset = Category.objects.all()
		resource_name = 'category'
		authentication = ApiKeyAuthentication()

class ElectionResource(ModelResource):
	candidates = fields.ToManyField(CandidateResource, 'candidate_set', null=True, full=False)
	categories = fields.ToManyField(CategoryResource, 'category_set', full=True)


	class Meta:
		queryset = Election.objects.all()
		resource_name = 'election'
		filtering = {"slug": ALL_WITH_RELATIONS, "name": ALL_WITH_RELATIONS }
		
		excludes = ['custom_style']
		authentication = ApiKeyAuthentication()

	def authorized_read_list(self, object_list, bundle):
		return object_list.filter(owner=bundle.request.user)

	def dehydrate(self, bundle):
		candidates_list = []
		for i, candidate in enumerate(bundle.obj.candidate_set.all()):
			candidates_list.append({'name':candidate.name, 'resource_uri':bundle.data['candidates'][i], 'id':candidate.id, 'image':candidate.photo})
		bundle.data['candidates'] = candidates_list
		current_site = Site.objects.get_current()
		bundle.data['url'] = "http://"+current_site.domain+bundle.obj.get_absolute_url()
		embeded_url = reverse('election_detail_embeded',kwargs={'username': bundle.obj.owner.username,'slug': bundle.obj.slug})
		bundle.data['embedded_url'] = "http://"+current_site.domain+embeded_url
		return bundle


from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_POST
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView

# Import forms
from elections.forms import BackgroundCategoryForm, BackgroundForm

# Import models
from elections.models import BackgroundCategory, Election


# BackgroundCategory Views
class BackgroundCategoryCreateView(CreateView):
    model = BackgroundCategory
    form_class = BackgroundCategoryForm
    
    def get_template_names(self):
        return ['elections/wizard/step_four.html']

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(BackgroundCategoryCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(BackgroundCategoryCreateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('background_category_create', kwargs={'election_slug': self.object.election.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election
        return super(BackgroundCategoryCreateView, self).form_valid(form)

@login_required
@require_POST
def async_delete_background_category(request, category_pk):
    category = get_object_or_404(BackgroundCategory, pk = category_pk, election__owner=request.user)
    category.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')


@login_required
@require_http_methods(['POST'])
def async_create_background_category(request, election_pk):
    election = get_object_or_404(Election, pk=election_pk, owner=request.user)

    value = request.POST.get('value', None)
    background_category = BackgroundCategory(name=value, election=election)
    background_category.save()
    return HttpResponse(json.dumps({"pk": background_category.pk, "name": background_category.name}),
                        content_type='application/json')



#encoding=UTF-8
from django import template
from django.utils.translation import ugettext as _
from elections.models import PersonalDataCandidate, BackgroundCandidate

register = template.Library()


@register.simple_tag
def value_for_candidate_and_personal_data(candidate, personal_data):
    '''
    Returns the answer for the given candidate and question pair.

    >> {% answer_for_candidate_and_question candidate question %}
    "answer"
    '''

    try:
        return PersonalDataCandidate.objects.get(candidate=candidate, personal_data=personal_data).value
    except PersonalDataCandidate.DoesNotExist:
        pass
    return ''

@register.simple_tag
def value_for_candidate_and_background(candidate, background):
    '''
    Returns the answer for the given candidate and question pair.

    >> {% answer_for_candidate_and_question candidate question %}
    "answer"
    '''

    try:
        return BackgroundCandidate.objects.get(candidate=candidate, background=background).value
    except BackgroundCandidate.DoesNotExist:
        pass
    return ''

# @register.simple_tag
# def candidate_photo(candidate):
#     '''
#     @deprecated
#     return the url for candidate photo or empty string
#     '''
#     try:
#         return candidate.photo.url
#     except ValueError, e:
#         pass
#     return ''




from answer_views import *
from background_views import *
from background_category_views import *
from candidate_views import *
from category_views import *
from election_views import *
from medianaranja_views import *
from personal_data_views import *
from question_views import *


from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.utils.unittest import skip

# Imported models
from elections.models import Election, Candidate, Category, Question, Answer, Visitor, VisitorAnswer, VisitorScore, CategoryScore
from django.contrib.auth.models import User

class TestMediaNaranja(TestCase):

    def setUp(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='election',
                                                            owner=user,
                                                            slug='barbaz')
        #deleting default categories
        for category in election.category_set.all():
            category.delete()
        #end of deleting default categories
        candidate1 = Candidate.objects.create(name='BarBaz', election=election)
        candidate2 = Candidate.objects.create(name='FooFoo', election=election)
        category1, created = Category.objects.get_or_create(name='FooCat',
                                                            election=election,
                                                            slug='foo-cat')
        category2, created = Category.objects.get_or_create(name='FooCat2',
                                                            election=election,
                                                            slug='foo-cat-2')
        question1, created = Question.objects.get_or_create(question='FooQuestion',
                                                            category=category1)
        question2, created = Question.objects.get_or_create(question='BarQuestion',
                                                            category=category2)
        answer1_1, created = Answer.objects.get_or_create(question=question1,
                                                        caption='BarAnswer1Question1')
        answer1_2, created = Answer.objects.get_or_create(question=question2,
                                                        caption='BarAnswer1Question2')
        answer2_1, created = Answer.objects.get_or_create(question=question1,
                                                        caption='BarAnswer2uestion1')
        answer2_2, created = Answer.objects.get_or_create(question=question2,
                                                        caption='BarAnswer2Question2')

        self.user = user
        self.election = election
        self.candidate1 = candidate1
        self.candidate2 = candidate2
        self.category1 = category1
        self.category2 = category2
        self.question1 = question1
        self.question2 = question2
        self.answer1_1 = answer1_1
        self.answer1_2 = answer1_2
        self.answer2_1 = answer2_1
        self.answer2_2 = answer2_2

        candidate1.associate_answer(self.answer1_1)
        candidate1.associate_answer(self.answer1_2)
        candidate2.associate_answer(self.answer2_1)
        candidate2.associate_answer(self.answer2_2)

    def test_reverse_routing_medianaranja1_correctly(self):
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        expected = "/joe/barbaz/medianaranja"
        self.assertEqual(url,expected)

    def test_answers_form(self):
        answers = [self.answer1_1.pk, self.answer1_2.pk]
        questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
        importances = [5, 3]
        importances_by_category = [5, 3]
        factor_question1 = (answers[0] == self.answer1_1.pk) * importances[0]
        factor_question2 = (answers[1] == self.answer1_2.pk) * importances[1]
        score_category1 = factor_question1 * 100.0 / importances_by_category[0]
        score_category2 = factor_question2 * 100.0 / importances_by_category[1]
        global_score = (factor_question1 + factor_question2) * 100.0 / sum(importances_by_category)
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        response = self.client.post(url, {'question-0': answers[0], 'question-1': answers[1], \
            'importance-0': importances[0], 'importance-1': importances[1],\
            'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})
        expected_winner = [global_score, [score_category1, score_category1], self.candidate1]
        self.assertEqual(response.context['winner'], expected_winner)

    def test_get_medianaranja(self):
        url = reverse("medianaranja1",kwargs={'username': self.user, 'election_slug':self.election.slug})
        response = self.client.get(url, {})
        self.assertEqual(response.status_code, 200)
        self.assertTrue('stt' in response.context)
        self.assertTrue('election' in response.context)
        self.assertEqual(self.election, response.context['election'])

    def test_post_to_nonexisting_user(self):
        url = reverse("medianaranja1",kwargs={'username': 'anonymoususer', 'election_slug':'barbaz'})
        response = self.client.post(url, {})
        self.assertEqual(response.status_code, 404)

    def test_get_number_of_questions_by_category(self):
        number_by_questions_expected = [1,1]
        number_by_questions = self.candidate1.get_number_of_questions_by_category()
        number_by_questions2 = self.candidate2.get_number_of_questions_by_category()
        self.assertEqual(number_by_questions_expected, number_by_questions)
        self.assertEqual(number_by_questions_expected, number_by_questions2)

    def test_get_importances_by_category(self):
        importances = [5, 3]
        importances_by_category_expected = [5,3]
        importances_by_category = self.candidate1.get_importances_by_category(importances)
        importances_by_category2 = self.candidate2.get_importances_by_category(importances)
        self.assertEqual(importances_by_category_expected, importances_by_category)
        self.assertEqual(importances_by_category_expected, importances_by_category2)

    def test_get_sum_importances_by_category(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [5, 3]
        sum_importances_by_category_expected = [5,3]
        sum_importances_by_category_expected2 = [0,0]
        sum_importances_by_category = self.candidate1.get_sum_importances_by_category(answers, importances)
        sum_importances_by_category2 = self.candidate2.get_sum_importances_by_category(answers, importances)
        self.assertEqual(sum_importances_by_category_expected, sum_importances_by_category)
        self.assertEqual(sum_importances_by_category_expected2, sum_importances_by_category2)

    def test_get_score(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        no_answers = [[], []]
        importances = [5, 3]
        get_score1 = self.candidate1.get_score(answers, importances)
        get_score2 = self.candidate2.get_score(answers, importances)
        get_score3 = self.candidate1.get_score(no_answers, importances)
        get_score4 = self.candidate2.get_score(no_answers, importances)
        get_score1_expected = (100.0, [100.0,100.0])
        get_score2_expected = (0, [0.0, 0.0])

        self.assertEqual(get_score1_expected, get_score1)
        self.assertEqual(get_score2_expected, get_score2)
        self.assertEqual(get_score2_expected, get_score3)
        self.assertEqual(get_score2_expected, get_score4)

    def test_get_score_with_zero_importances(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [0, 0]

        get_score = self.candidate1.get_score(answers, importances)
        expected_score = (0, [0.0, 0.0])

        self.assertEqual(expected_score,get_score)

    def test_save_new_visitor(self):
        answers = [self.answer1_1.pk, self.answer1_2.pk]
        questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        response = self.client.post(url, {'question-0': answers[0], 'question-1': answers[1], \
            'importance-0': importances[0], 'importance-1': importances[1],\
            'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})
        visitors = Visitor.objects.all()
        self.assertEqual(visitors.count(), 1)
        me = visitors[0]
        self.assertEqual(me.election,self.election)
        self.assertEqual(me.election_url, reverse("election_detail",kwargs={'username': 'joe', 'slug':'barbaz'}))
        # TODO test unique hash
        # TODO test datestamp

    def test_create_visitor_answer(self):
        visitor1 = Visitor.objects.create(election = self.election, election_url = reverse("election_detail",kwargs={'username': 'joe', 'slug':'barbaz'}))
        visitor_answer1= VisitorAnswer.objects.create(visitor=visitor1, answer= self.answer1_1, answer_importance=5)
        self.assertEqual(visitor_answer1.answer_text, self.answer1_1.caption)
        self.assertEqual(visitor_answer1.question_text, self.answer1_1.question.question)
        self.assertEqual(visitor_answer1.question_category_text, self.answer1_1.question.category.name)
        self.assertEqual(visitor_answer1.answer_importance, 5)


    def test_save_visitors_answers(self):
        answers = [self.answer1_1, self.answer1_2]
        questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        response = self.client.post(url, {'question-0': answers[0].pk, 'question-1': answers[1].pk, \
            'importance-0': importances[0], 'importance-1': importances[1],\
            'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})
        visitors = Visitor.objects.all()
        me = visitors[0]
        visitors_answers = me.visitoranswer_set.all()
        self.assertEqual(visitors_answers.count(), 2)
        self.assertEqual(visitors_answers[0].answer_importance, importances[0])
        self.assertEqual(visitors_answers[1].answer_importance, importances[1])
        self.assertEqual(visitors_answers[0].answer_text, answers[0].caption)
        self.assertEqual(visitors_answers[1].answer_text, answers[1].caption)
        self.assertEqual(visitors_answers[1].question_text, answers[1].question.question)
        self.assertEqual(visitors_answers[0].question_text, answers[0].question.question)
        self.assertEqual(visitors_answers[0].question_text, answers[0].question.question)
    
    def test_save_visitors_answers_default_values(self):
        questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
        answers = [-1, -1]
        importances = [3, 3]
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        response = self.client.post(url, {'question-0': answers[0], 'question-1': answers[1], \
            'importance-0': importances[0], 'importance-1': importances[1], \
            'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})
        visitors = Visitor.objects.all()
        me = visitors[0]
        visitors_answers = me.visitoranswer_set.all()
        self.assertEqual(visitors_answers.count(), 2)
        self.assertEqual(visitors_answers[0].answer_importance, importances[0])
        self.assertEqual(visitors_answers[1].answer_importance, importances[1])
        self.assertEqual(visitors_answers[0].answer_text, "")
        self.assertEqual(visitors_answers[1].answer_text, "")
        self.assertEqual(visitors_answers[0].question_text, self.answer1_1.question.question)
        self.assertEqual(visitors_answers[1].question_text, self.answer1_2.question.question)


    def test_create_visitor_score(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        categories = [answers[0][0].question.category.name, answers[1][0].question.category.name]
        importances = [5, 3]
        get_score1 = self.candidate1.get_score(answers, importances)
        visitor1 = Visitor.objects.create(election = self.election, election_url = reverse("election_detail",kwargs={'username': 'joe', 'slug':'barbaz'}))
        visitor_score = VisitorScore.objects.create(visitor=visitor1,candidate_name=self.candidate1.name,score=get_score1[0])
        category_score1 = CategoryScore.objects.create(visitor_score=visitor_score,category_score=get_score1[1][0],category_name=categories[0])
        category_score2 = CategoryScore.objects.create(visitor_score=visitor_score,category_score=get_score1[1][1],category_name=categories[1])
        self.assertEqual(visitor_score.score, get_score1[0])
        self.assertEqual(visitor_score.candidate_name, self.candidate1.name)
        visitor_category_scores = visitor_score.categoryscore_set.all()
        self.assertEqual(visitor_category_scores.count(), 2)

    def test_save_visitor_score(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': 'joe', 'election_slug':'barbaz'})
        response = self.client.post(url, {'question-0': answers[0][0].pk, 'question-1': answers[1][0].pk, \
            'importance-0': importances[0], 'importance-1': importances[1], \
            'question-id-0': answers[0][0].question.pk , 'question-id-1': answers[1][0].question.pk})
        visitors = Visitor.objects.all()
        me = visitors[0]
        visitor_scores = me.visitorscore_set.all()
        visitor_candidate1_category_scores = CategoryScore.objects.filter(visitor_score=visitor_scores.get(candidate_name=self.candidate1.name))
        visitor_candidate2_category_scores = CategoryScore.objects.filter(visitor_score=visitor_scores.get(candidate_name=self.candidate2.name))
        scores_candidate1 = self.candidate1.get_score(answers, importances)
        scores_candidate2 = self.candidate2.get_score(answers, importances)

        self.assertEqual(visitor_scores.filter(candidate_name=self.candidate1.name).count(), 1)
        self.assertEqual(visitor_scores.filter(candidate_name=self.candidate2.name).count(), 1)
        self.assertEqual(visitor_scores.get(candidate_name=self.candidate1.name).candidate_name,self.candidate1.name) 
        self.assertEqual(visitor_scores.get(candidate_name=self.candidate2.name).candidate_name,self.candidate2.name) 
        self.assertEqual(visitor_scores.get(candidate_name=self.candidate1.name).score,scores_candidate1[0]) 
        self.assertEqual(visitor_scores.get(candidate_name=self.candidate2.name).score,scores_candidate2[0])
        self.assertEqual(visitor_candidate1_category_scores.get(category_name=answers[0][0].question.category.name).category_score ,scores_candidate1[1][0])
        self.assertEqual(visitor_candidate1_category_scores.get(category_name=answers[1][0].question.category.name).category_score ,scores_candidate1[1][1])
        self.assertEqual(visitor_candidate2_category_scores.get(category_name=answers[0][0].question.category.name).category_score ,scores_candidate2[1][0])
        self.assertEqual(visitor_candidate2_category_scores.get(category_name=answers[1][0].question.category.name).category_score ,scores_candidate2[1][1])
     



class TestMediaNaranjaWithNoCategories(TestCase):


    def setUp(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='election',
            owner=user,
            slug='barbaz')
        #deleting default categories
        for category in election.category_set.all():
            category.delete()
        #end of deleting default categories
        self.candidate1 = Candidate.objects.create(name='BarBaz', election=election)
        self.candidate2 = Candidate.objects.create(name='FooFoo', election=election)



    def test_get_score_without_categories(self):
        answers = []
        importances = []
        get_score = self.candidate1.get_score(answers,importances)
        expected_score = (0,[])
        self.assertEqual(expected_score,get_score)



class TestElectionsWithoutDefaultOptionInMediaNaranja(TestCase):

    def setUp(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='election',
                                                            owner=user,
                                                            slug='barbaz')
        #deleting default categories
        for category in election.category_set.all():
            category.delete()
        #end of deleting default categories
        candidate1 = Candidate.objects.create(name='BarBaz', election=election)
        candidate2 = Candidate.objects.create(name='FooFoo', election=election)
        category1, created = Category.objects.get_or_create(name='FooCat',
                                                            election=election,
                                                            slug='foo-cat')
        category2, created = Category.objects.get_or_create(name='FooCat2',
                                                            election=election,
                                                            slug='foo-cat-2')
        question1, created = Question.objects.get_or_create(question='FooQuestion',
                                                            category=category1)
        question2, created = Question.objects.get_or_create(question='BarQuestion',
                                                            category=category2)
        answer1_1, created = Answer.objects.get_or_create(question=question1,
                                                        caption='BarAnswer1Question1')
        answer1_2, created = Answer.objects.get_or_create(question=question2,
                                                        caption='BarAnswer1Question2')
        answer2_1, created = Answer.objects.get_or_create(question=question1,
                                                        caption='BarAnswer2uestion1')
        answer2_2, created = Answer.objects.get_or_create(question=question2,
                                                        caption='BarAnswer2Question2')

        self.user = user
        self.election = election
        self.candidate1 = candidate1
        self.candidate2 = candidate2
        self.category1 = category1
        self.category2 = category2
        self.question1 = question1
        self.question2 = question2
        self.answer1_1 = answer1_1
        self.answer1_2 = answer1_2
        self.answer2_1 = answer2_1
        self.answer2_2 = answer2_2

        candidate1.associate_answer(self.answer1_1)
        candidate1.associate_answer(self.answer1_2)
        candidate2.associate_answer(self.answer2_1)
        candidate2.associate_answer(self.answer2_2)



    def test_an_election_should_have_default_option_medianaranja_field(self):

        self.assertTrue(self.election.use_default_media_naranja_option)


    def test_media_naranja_get_403_if_no_csrf_token_sent(self):
        from django.test import Client
        csrf_client = Client(enforce_csrf_checks=True)


        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': self.election.owner.username, 'election_slug': self.election.slug })
        data_to_be_posted = {
                            'question-0': answers[0][0].pk, 
                            #'question-1': -1, #THIS ANSWER IS NOT BEING PASSED AND IT WILL NOT COUNT
                            'importance-0': importances[0], 
                            'importance-1': importances[1], 
                            'question-id-0': answers[0][0].question.pk , 
                            'question-id-1': answers[1][0].question.pk
                            }
        response = csrf_client.post(url, data_to_be_posted)
        expected_winner = [62.5 , [100, 0.0], self.candidate1]


        self.assertEqual(response.status_code, 403)



    def test_media_naranja_could_be_answered_without_one_options(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': self.election.owner.username, 'election_slug': self.election.slug })
        data_to_be_posted = {
                            'question-0': answers[0][0].pk, 
                            #'question-1': -1, #THIS ANSWER IS NOT BEING PASSED AND IT WILL NOT COUNT
                            'importance-0': importances[0], 
                            'importance-1': importances[1], 
                            'question-id-0': answers[0][0].question.pk , 
                            'question-id-1': answers[1][0].question.pk
                            }
        response = self.client.post(url, data_to_be_posted)
        expected_winner = [62.5 , [100, 0.0], self.candidate1]




        self.assertEquals(response.context['winner'], expected_winner)


    def test_media_naranja_without_any_options(self):
        answers = [[self.answer1_1], [self.answer1_2]]
        importances = [5, 3]
        url = reverse("medianaranja1",kwargs={'username': self.election.owner.username, 'election_slug': self.election.slug })
        data_to_be_posted = {
                            #'question-0': answers[0][0].pk, 
                            #'question-1': -1, #THIS ANSWER IS NOT BEING PASSED AND IT WILL NOT COUNT
                            'importance-0': importances[0], 
                            'importance-1': importances[1], 
                            'question-id-0': answers[0][0].question.pk , 
                            'question-id-1': answers[1][0].question.pk
                            }
        response = self.client.post(url, data_to_be_posted)
        expected_winner = [0.0 , [0.0, 0.0]]


        self.assertEquals(response.context['winner'][0], expected_winner[0])
        self.assertEquals(response.context['winner'][1], expected_winner[1])



# coding= utf-8
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from elections.models import Election, Candidate, PersonalData, Category, Question, Answer, BackgroundCategory,\
							 Background, Link, BackgroundCandidate, PersonalDataCandidate, InformationSource
import csv
from django.core.urlresolvers import reverse
from django.db.models import Q

class QuestionsParser(object):
	def __init__(self, election):
		self.election = election
		self.election.backgroundcategory_set.all().delete()
		self.election.personaldata_set.all().delete()

	def createQuestions(self, lines):
		for line in lines:
			if line[0]=="category":
				category_name = line[1].decode('utf-8').strip()
				self.category = Category.objects.create(name=category_name, election=self.election)
			if line[0]=="question":
				question = line[1].decode('utf-8').strip()
				self.question = Question.objects.create(question=question, category=self.category)
			if line[0]=="answer":
				answer = line[1].decode('utf-8').strip()
				self.answer = Answer.objects.create(caption=answer, question=self.question)
			if line[0]=="background history category":
				background_category_name = line[1].decode('utf-8').strip()
				self.background_category = BackgroundCategory.objects.create(name=background_category_name, election=self.election)
			if line[0]=="background history":
				background_history_name = line[1].decode('utf-8').strip()
				self.background_history = Background.objects.create(name=background_history_name, category=self.background_category)
			if line[0]=="personal data":
				personal_data_label = line[1].decode('utf-8').strip()
				self.personal_data = PersonalData.objects.create(label=personal_data_label, election=self.election)

class AnswersLoader(object):
	def __init__(self,username, candidates, questions, styles):
		self.user = User.objects.get(username=username)
		self.lines = candidates
		self.styles = styles
		self.questions = questions
		self.get_definitions()

	def get_definitions(self):
		line0 = self.lines[0]
		line1 = self.lines[1]
		self.definitions = {}
		if len(line0) > 2:
			for i in range(2, len(line0)):
				definition = {"label":line1[i], "type":line0[i]}
				self.definitions[i] = definition


	def get_election(self, line, questions):
		election_name = line[0].decode('utf-8').strip()
		election, created = Election.objects.get_or_create(name=election_name, owner=self.user)

		if(created):
		 	election.custom_style = self.styles
		 	election.save()
		 	election.category_set.all().delete()
		 	parser = QuestionsParser(election)
			parser.createQuestions(questions)
		return election

	def assign_values(self, election, candidate, line):
		for i in range(2,len(line)):
			value = line[i].decode('utf-8').strip()
			label = self.definitions[i]["label"]
			the_type = self.definitions[i]["type"]

			if(the_type == "personal data"):
				try:
					personal_data = PersonalData.objects.get(election=election, label=label)
					PersonalDataCandidate.objects.create(candidate=candidate, personal_data=personal_data, value=value)
				except :
					print "personal data non existent "+label +"| value "+ value
			if (the_type == "background history"):
				try:

					category_name = label.split(" - ")[0]
					label = label.split(" - ")[1]
					category = BackgroundCategory.objects.get(name=category_name, election=election)
					background = Background.objects.get(name=label, category=category)
					BackgroundCandidate.objects.create(candidate=candidate, background=background, value=value)
				except :
					print "background non existent "+label +"| value "+ value

			if (the_type == "link"):
				try:
					if label in ["twitter","facebook"]:
						if (label == "twitter"):
							Link.objects.create(name=u'@'+value, url=u"https://twitter.com/"+value, candidate=candidate)
						if (label == "facebook"):
							Link.objects.create(name=candidate.name, url=value, candidate=candidate)
					else:
						if value:
							Link.objects.create(name=label, url=value, candidate=candidate)

				except:
					print "link non existent "+label +"| value "+ value


			if (the_type == "answer"):
				pre_processed_answer = value.split("<")
				answer_caption = pre_processed_answer.pop(0)
				answer_caption = answer_caption.strip()
				#here there is a bug
				question = Question.objects.get(Q(question=label) & Q(category__election=election))
				#yes here above this
				try:
					answer = Answer.objects.get(Q(question__category__election=election) & Q(question=question) & Q(caption=answer_caption))
					candidate.associate_answer(answer)
					if pre_processed_answer:
						information_source_content = pre_processed_answer[0][:-1]
						InformationSource.objects.create(question=answer.question, candidate=candidate, content=information_source_content)
				except Answer.DoesNotExist, exception:
					if answer_caption:
						print u"The answer '%(answer)s' for question '%(question)s' and candidate %(candidate)s is not defined in the questions csv"%{
						'answer':answer_caption,
						'question':question.question,
						'candidate':candidate.name
						}
				except Answer.MultipleObjectsReturned, exception:
					print u"The answer '%(answer)s' for question '%(question)s' is defined twice in the questions csv"%{
						'answer':answer_caption,
						'question':question.question
						}


	def get_candidate(self, line, election):
		candidate = Candidate.objects.create(name=line[1].decode('utf-8').strip(), election=election)
		return candidate

	def process(self):
		amount_of_lines = len(self.lines)

		for i in range(2, amount_of_lines):
			line = self.lines[i]
			election = self.get_election(line, self.questions)
			candidate = self.get_candidate(line, election)
			self.assign_values(election, candidate, line)




class Command(BaseCommand):
	args = '<username> <candidates csv file> <questions csv file> <css file for custom>'
	def handle(self, *args, **options):
		username = args[0]
		reader = open(args[3], 'rb')
		style = reader.read()
		lines_for_candidates_loader = csv.reader(open(args[1], 'rb'), delimiter=',')
		candidates =[]
		for line in lines_for_candidates_loader:
			candidates.append(line)
		questions_reader = csv.reader(open(args[2], 'rb'), delimiter=',')
		questions = []
		for line in questions_reader:
			questions.append(line)
		loader = AnswersLoader(username, candidates, questions, style)
		loader.process()


#encoding=UTF-8
from django import template
from django.conf import settings

register = template.Library()

@register.simple_tag
def uservoice_client_key():
    return settings.USERVOICE_CLIENT_KEY


@register.simple_tag
def ga_account_id():
    return settings.GOOGLE_ANALYTICS_ACCOUNT_ID


# encoding=UTF-8
# Django settings for candidator project.
import os

DEBUG = True
TEMPLATE_DEBUG = DEBUG

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

INTERNAL_IPS = ('127.0.0.1',)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': os.path.join(PROJECT_ROOT, 'development.db'),                      # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Santiago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'es-cl'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = os.path.join(PROJECT_ROOT, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(PROJECT_ROOT, 'public')


# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

USER_FILES = 'static-files'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = STATIC_URL + "grappelli/"

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '&1fs&w#=61f3iokgh_lef4%thd10um4q2d&a)orl1%!sc0@8le'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
    'django.core.context_processors.request',
)


ROOT_URLCONF = 'candidator.urls'

TEMPLATE_DIRS = (
    os.path.join(PROJECT_ROOT, 'templates'),
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    'grappelli',
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    'django.contrib.admindocs',
    'debug_toolbar',
    'django_coverage',
    'sorl.thumbnail',
    'profiles',
    'smart_profile',
    'elections',
    'report_objects',
    'django_extensions',
    # Django-registration
    'registration',
    # Tastypie, RESTful APIs for Django:
    'tastypie',
    'markdown_deux',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

DEBUG_TOOLBAR_PANELS = (
    'debug_toolbar.panels.version.VersionDebugPanel',
    'debug_toolbar.panels.timer.TimerDebugPanel',
    'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
    'debug_toolbar.panels.headers.HeaderDebugPanel',
    'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
    'debug_toolbar.panels.template.TemplateDebugPanel',
    'debug_toolbar.panels.sql.SQLDebugPanel',
    'debug_toolbar.panels.signals.SignalDebugPanel',
    'debug_toolbar.panels.logger.LoggingPanel',
)

DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
    'HIDE_DJANGO_SQL': False,
}

AUTH_PROFILE_MODULE = 'smart_profile.Profile'

# Redirect to this url after login
LOGIN_REDIRECT_URL = '/'

# Account activation days, one-week window
ACCOUNT_ACTIVATION_DAYS = 7

# Serialization modules
SERIALIZATION_MODULES = {'json-pretty': 'serializers.json_pretty'}

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

DEFAULT_PERSONAL_DATA = ['Edad', 'Estado civil', u'Profesión', u'Género']
DEFAULT_BACKGROUND_CATEGORIES = {u'Educación': [u'Educación primaria', u'Educación secundaria', u'Educación superior'],
                                 u'Antecedentes laborales': [u'Último trabajo']}

DEFAULT_QUESTIONS = [{
    'Category':u'Educación',
    'Questions':[
        {
            'question':u'¿Crees que Chile debe tener una educación gratuita?',
            'answers':[u'Sí',u'No']
        },
        {
            'question':u'¿Estas de acuerdo con la desmunicipalización?',
            'answers':[u'Sí',u'No']
        }
    ]
}]


USERVOICE_CLIENT_KEY = 'THIS IS JUST AN EXAMPLE YOU SHOLD CHANGE THIS'
GOOGLE_ANALYTICS_ACCOUNT_ID = "GOOGLE ANALYTICS ACCOUNT ID"


#EMBEDED WEBPAGE FOR TESTING

EMBEDED_TEST_WEB = 'http://localhost:8000/admin/cei-2012/embeded'

try:
    from local_settings import *
except ImportError:
    pass



from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404, HttpResponseForbidden, HttpResponseBadRequest
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json, simplejson
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView
from django.views.generic.edit import BaseDeleteView

# Import forms
from elections.forms import AnswerForm

# Import models
from elections.models import Answer, Question


class AnswerCreateAjaxView(CreateView):
    model = Answer
    form_class = AnswerForm

    def dispatch(self, request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponseBadRequest()
        self.question = get_object_or_404(Question, pk=kwargs['question_pk'])
        if self.question.category.election.owner != request.user:
            return HttpResponseForbidden()
        return super(AnswerCreateAjaxView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.question = self.question
        self.object.save()
        return HttpResponse(content=simplejson.dumps(
                {'pk': self.object.pk, 'caption': self.object.caption, 'question': self.question.pk}),
                content_type='application/json')

    def form_invalid(self, form):
        return HttpResponse(content=simplejson.dumps(
            {'error': form.errors}),
            content_type='application/json')


# Answer View
class AnswerCreateView(CreateView):
    model = Answer
    form_class = AnswerForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(AnswerCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(AnswerCreateView, self).get_context_data(**kwargs)
        context['question'] = get_object_or_404(Question, pk=self.kwargs['question_pk'], category__election__owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('category_create', kwargs={'election_slug': self.object.question.category.election.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        question = get_object_or_404(Question, pk=self.kwargs['question_pk'], category__election__owner=self.request.user)
        self.object.question = question
        return super(AnswerCreateView, self).form_valid(form)



class AnswerDeleteAjaxView(BaseDeleteView):
    model = Answer
    pk_url_kwarg = 'pk'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(AnswerDeleteAjaxView, self).dispatch(request, *args, \
                **kwargs)

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object.question.category.election.owner != request.user:
            raise Http404
        self.object.delete()
        json_dictionary = {"result":"OK"}
        return HttpResponse(json.dumps(json_dictionary),content_type='application/json')





from django.test import TestCase
from django.template import Template, Context
from django.conf import settings

class SettingsVariablesInTemplate(TestCase):
	def setUp(self):
		pass

	def test_it_provides_uservoice_template_tag(self):
		settings.USERVOICE_CLIENT_KEY = "USERVOICE KEY"
		template = Template('{% load settingsvars_tags %}{% uservoice_client_key %}')
		self.assertEqual(template.render(Context({})), "USERVOICE KEY")

	def test_it_provides_google_analytics_template_tag(self):
		settings.GOOGLE_ANALYTICS_ACCOUNT_ID = "GOOGLE ANALYTICS ACCOUNT ID"
		template = Template('{% load settingsvars_tags %}{% ga_account_id %}')
		self.assertEqual(template.render(Context({})), "GOOGLE ANALYTICS ACCOUNT ID")


from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404, HttpResponseForbidden, HttpResponseBadRequest
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json, simplejson
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_POST, require_GET
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView, ListView
from django.utils.translation import ugettext_lazy as _


# Import forms
from elections.forms import CandidateForm, CandidateUpdateForm, CandidateLinkForm, BackgroundCandidateForm, AnswerForm, PersonalDataCandidateForm, CandidatePhotoForm

# Import models
from elections.models import Election, Candidate, PersonalData, Link

# Import exceptions
from elections.exceptions import NoCandidateError

# Candidate views
class CandidateDetailView(DetailView):
    model = Candidate

    def get_context_data(self, **kwargs):
        context = super(CandidateDetailView, self).get_context_data(**kwargs)
        context['election'] = self.object.election
        return context

    def get_queryset(self):
        return self.model.objects.filter(election__owner__username=self.kwargs['username'],
                                         election__slug=self.kwargs['election_slug'],
                                         slug=self.kwargs['slug'])


class CandidateUpdateView(UpdateView):
    model = Candidate
    form_class = CandidateUpdateForm
    
    def get_template_names(self):
        return ['elections/wizard/step_two.html']
    
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(CandidateUpdateView, self).dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.model.objects.filter(election__slug=self.kwargs['election_slug'],
                                          slug=self.kwargs['slug'],
                                          election__owner=self.request.user)

    def get_context_data(self, **kwargs):
        context = super(CandidateUpdateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('candidate_update', kwargs={'slug': self.kwargs['slug'], 'election_slug': self.object.election.slug})


class CandidateCreateView(CreateView):
    model = Candidate
    form_class = CandidateForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(CandidateCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(CandidateCreateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        url_to_updating_candidates = reverse('candidate_create_alone', kwargs={'election_slug': self.object.election.slug})
        if self.request.path_info == url_to_updating_candidates:
            return url_to_updating_candidates
        return reverse('candidate_create', kwargs={'election_slug': self.object.election.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election
        try:
            self.object.full_clean()
            self.object.save()
        except ValidationError as e:
            from django.forms.util import ErrorList
            form._errors["nombre"] = ErrorList([u"Ya tienes un candidato con ese nombre."])
            return super(CandidateCreateView, self).form_invalid(form)
        return super(CandidateCreateView, self).form_valid(form)



class CandidateDataUpdateView(UpdateView):
    model = Candidate
    form_class = CandidateForm
    first_time = False
                
    @method_decorator(login_required)
    @method_decorator(require_GET)
    def dispatch(self, request, *args, **kwargs):
        try:
            return super(CandidateDataUpdateView, self).dispatch(request, *args, **kwargs)
        except NoCandidateError:
            return redirect("candidate_create_alone", election_slug=self.kwargs['election_slug'])


    def get_queryset(self):
        try:
            election = Election.objects.get(slug=self.kwargs['election_slug'],owner=self.request.user)
        except:
            return Election.objects.none()

        if 'slug' not in self.kwargs:
            candidates = election.candidate_set.all()
            if not candidates:
                raise NoCandidateError(_(u"There are no candidates"),[_(u"There are no candidates")])
            self.kwargs['slug'] = candidates[0].slug

        return self.model.objects.filter(election=election,
                                          slug=self.kwargs['slug'])



    def get_context_data(self, **kwargs):
        context = super(CandidateDataUpdateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        context['candidate_link_form'] = CandidateLinkForm()
        context['background_candidate_form'] = BackgroundCandidateForm()
        context['personal_data_candidate_form'] = PersonalDataCandidateForm()
        context['answer_form'] = AnswerForm()
        context['first_time'] = self.first_time
        return context


@login_required
@require_POST
def async_delete_candidate(request):
    candidate_pk = request.POST['candidate_pk']
    candidate = get_object_or_404(Candidate, pk=candidate_pk, election__owner=request.user)
    candidate.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')


@require_POST
def get_candidate_list_as_json(request,username,election_slug):
    election = get_object_or_404(Election, owner__username=username, slug=election_slug)
    candidates = election.candidate_set.all()
    result = []
    for candidate in candidates:
        result.append({'name' : candidate.name,'id' : candidate.id})
    return HttpResponse(json.dumps(result),content_type='application/json')

class CandidateCreateAjaxView(CreateView):
    model = Candidate
    form_class = CandidateForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponseBadRequest()
        self.election = get_object_or_404(Election, owner=request.user, slug=kwargs['election_slug'])
        return super(CandidateCreateAjaxView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election
        try:
            self.object.full_clean()
            self.object.save()
        except ValidationError as e:
            from django.forms.util import ErrorList
            form._errors["slug"] = ErrorList([u"Ya tienes un candidato con ese slug."])
            return HttpResponse(content=simplejson.dumps(
                {'error': form.errors}),
                content_type='application/json')

        self.object = form.save(commit=False)
        self.object.election = self.election
        self.object.save()
        json_dictionary = {"result": "OK"}
        return HttpResponse(json.dumps(json_dictionary),content_type='application/json')

    def form_invalid(self, form):
        return HttpResponse(content=simplejson.dumps({'error': form.errors}), content_type='application/json')

@login_required
@require_POST
def async_create_link(request, candidate_pk):
    link_name = request.POST.get('link_name', False)
    link_url = request.POST.get('link_url', False)
    election_slug = request.POST.get('election_slug', False)
    slug = request.POST.get('candidate_slug', False)
    cand_name = request.POST.get('candidate_name', False)
    election = get_object_or_404(Election, slug=election_slug, owner=request.user)
    candidate = get_object_or_404(Candidate, slug=slug, election=election, name=cand_name)
    link = Link(name = link_name, url= link_url, candidate = candidate)
    try:
        link.full_clean()
        link.save()
        json_dictionary = {"result": "OK", 'name':link_name, 'url':link.http_prefix, 'pk':link.pk}
    except ValidationError as e:
        json_dictionary = {"result":"NOK", "errors":e.message_dict}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')

   

@login_required
@require_POST
def async_delete_link(request, link_pk):
    link = get_object_or_404(Link, pk = link_pk, candidate__election__owner = request.user)
    link.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')

class CandidateUpdatePhotoView(UpdateView):
    form_class = CandidatePhotoForm
    model = Candidate

    def get_template_names(self):
        return 'elections/candidate_photo_form.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if Candidate.objects.filter(pk=kwargs['pk'], election__owner=request.user).count() <= 0:
            return HttpResponseForbidden()
        return super(CandidateUpdatePhotoView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(CandidateUpdatePhotoView, self).get_form_kwargs()
        kwargs['candidate'] = self.object
        return kwargs

    def get_success_url(self):
        return reverse('candidate_data_update',
                       kwargs={'election_slug': self.object.election.slug, 'slug': self.object.slug})




"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase
from django.contrib.auth.models import User

from models import Profile

class SimpleTest(TestCase):
    def test_basic_addition(self):
        user = User.objects.create(username='foo', password='foo', email='foo@example.net')
        self.assertEqual(Profile.objects.count(), 1)
        profile = Profile.objects.get()
        self.assertEqual(profile.user, user)
        self.assertEqual(profile.name, user.username)



# -*- coding: utf-8 -*-


from django.test import TestCase
from elections.models import InformationSource
from django.contrib.auth.models import User
from elections.models import Candidate, Election, Category, Question, Answer
from django.db.models import Q
from django.core.exceptions import ValidationError

class InformationSourceTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='doe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.not_user = User.objects.create_user(username='joe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                            owner=self.user,
                                                            slug='barbaz2')
        self.candidate = Candidate.objects.create(name='Bar Baz',
                                                            election=self.election)
        self.candidate2, created = Candidate.objects.get_or_create(
                                                            name='Bar Baz',
                                                            election=self.election2)
        categories = [
            Category.objects.get_or_create(election=self.election,
                                            name='Cat1',
                                            slug='cat1'),
            Category.objects.get_or_create(election=self.election,
                                            name='Cat2',
                                            slug='cat2'),
        ]
        self.categories = [cat for cat, created in categories]
        category2, created = Category.objects.get_or_create(election=self.election2,
                                                     name='Cat2')
        self.question, created = Question.objects.get_or_create(question='Foo',
                                                            category=self.categories[0])
        question2, created = Question.objects.get_or_create(question='Foo',
                                                            category=category2)
        self.answer, created = Answer.objects.get_or_create(question=self.question,
                                                       caption='Bar')
        self.answer2, created = Answer.objects.get_or_create(question=question2,
                                                       caption='Bar')


        self.candidate.associate_answer(self.answer2)

    def test_create_an_information_source(self):

        information_source = InformationSource.objects.create(
            question=self.question, 
            candidate=self.candidate, 
            content=u'We saw this answer at el mostrador')

        self.assertTrue(information_source)
        self.assertEquals(information_source.question, self.question)
        self.assertEquals(information_source.candidate, self.candidate)
        self.assertEquals(information_source.content, u'We saw this answer at el mostrador')



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth.models import User
from django.db import models
from tastypie.models import ApiKey
from elections.models import Election, Candidate, Category, PersonalData, \
                             BackgroundCategory, Background, PersonalDataCandidate, \
                             Question, Answer, Link, BackgroundCandidate
from elections.api import BackgroundResource
from tastypie.test import ResourceTestCase, TestApiClient
from django.core import serializers
from django.core.urlresolvers import reverse
from django.utils.unittest import skip
from django.contrib.sites.models import Site

class ApiTestCase(ResourceTestCase):
    
    def setUp(self):
        super(ApiTestCase, self).setUp()
        self.user = User.objects.create_user(username='the_user',
                                                password='joe',
                                                email='doe@joe.cl')
        Election.objects.filter(owner=self.user).delete()
        self.not_user = User.objects.create_user(username='joe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz',
                                                            published=True)
        self.election.category_set.all().delete()
        self.election.personaldata_set.all().delete()
        self.personal_data1 = PersonalData.objects.create(label=u"Age", election=self.election)
        self.personal_data2 = PersonalData.objects.create(label=u"Profession", election=self.election)

        self.category1 = Category.objects.create(name=u"Pets and phisicians", election=self.election, slug="pets")
        self.category2 = Category.objects.create(name=u"language problemas ", election=self.election, slug="language")
        self.question1 = Question.objects.create(category=self.category1, question=u"¿Cuál es el nombre de la ferocidad?")
        self.answer1 = Answer.objects.create(caption=u"Fiera", question=self.question1)
        self.answer2 = Answer.objects.create(caption=u"Ratón inteligente pero barza", question=self.question1)
        self.question2 = Question.objects.create(category=self.category1, question=u"¿Which one is your favourite colour?")
        self.answer3 = Answer.objects.create(caption=u"apple", question=self.question2)
        self.answer4 = Answer.objects.create(caption=u"orange", question=self.question2)
        self.question3 = Question.objects.create(category=self.category2, question=u"¿!¿Why don't you speak proper english?!?")
        self.answer5 = Answer.objects.create(caption=u"Hablo inglés perfectamente", question=self.question3)
        self.answer6 = Answer.objects.create(caption=u"I don't speak any english", question=self.question3)

        self.election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                            owner=self.not_user,
                                                            slug='barbaz2',
                                                            published=True)
        self.candidate = Candidate.objects.create(
                                                            name='Bar Baz',
                                                            election=self.election)
        self.candidate.associate_answer(self.answer2)

        self.age1 = PersonalDataCandidate.objects.create(personal_data=self.personal_data1, candidate=self.candidate, value=u"2")
        self.profession1 = PersonalDataCandidate.objects.create(personal_data=self.personal_data2, candidate=self.candidate, value=u"Perro")
        self.candidate2 = Candidate.objects.create(
                                                            name='Fieri',
                                                            election=self.election)
        self.candidate3  = Candidate.objects.create(
                                                            name='Ratón 1',
                                                            election=self.election2)
        self.candidate3.personal_data.all().delete()
        self.election3, created = Election.objects.get_or_create(name='BarBaz3',
                                                            owner=self.user,
                                                            slug='barbaz3',
                                                            published=True)
        self.ferocious_link, created = Link.objects.get_or_create(
                                                    name='@fiera',
                                                    url='http://www.twitter.com/fiera',
                                                    candidate=self.candidate2)
        self.api_client = TestApiClient()
        self.data = {'format': 'json', 'username': self.user.username, 'api_key':self.user.api_key.key}


    def test_get_all_my_elections(self):
        url = '/api/v1/election/'
        resp = self.api_client.get(url,data = self.data)
        self.assertValidJSONResponse(resp)
        elections = self.deserialize(resp)['objects']
        self.assertEqual(len(elections), 2) #Only my elections
        self.assertTrue(elections[0].has_key("name"))
        self.assertEqual(elections[0]["id"], self.election.id) #I make sure this is the election
        self.assertTrue(elections[0].has_key("id"))
        self.assertTrue(elections[0].has_key("slug"))
        self.assertTrue(elections[0].has_key("description"))
        self.assertTrue(elections[0].has_key("information_source"))
        self.assertTrue(elections[0].has_key("logo"))
        self.assertTrue(elections[0].has_key("published"))
        self.assertFalse(elections[0].has_key("custom_style"))
        self.assertTrue(elections[0].has_key("candidates"))
        self.assertTrue(elections[0]['candidates'][0].has_key("id"))
        self.assertTrue(elections[0]['candidates'][0].has_key("name"))
        self.assertTrue(elections[0]['candidates'][0].has_key("resource_uri"))
        self.assertTrue(elections[0]['candidates'][0].has_key("image"))
        # self.assertTrue(elections[0]['candidates'][0].has_key("questions"))
        # self.assertTrue(elections[0]['candidates'][0]['questions'][0].has_key("question"))
        current_site = Site.objects.get_current()
        self.assertTrue(elections[0].has_key("url"))
        self.assertEquals(elections[0]["url"], "http://"+current_site.domain+self.election.get_absolute_url())
        self.assertTrue(elections[0].has_key("embedded_url"))
        embeded_url = reverse('election_detail_embeded',kwargs={'username': self.user.username,'slug': self.election.slug})
        self.assertEquals(elections[0]["embedded_url"], "http://"+current_site.domain+embeded_url)

    def test_get_filter_elections(self):
        url = '/api/v1/election/'
        filter_key = 'slug'
        filter_value = 'barbaz'
        self.data[filter_key] = filter_value
        resp = self.api_client.get(url,data = self.data)
        self.assertValidJSONResponse(resp)
        elections = self.deserialize(resp)['objects']
        self.assertEqual(len(elections), 1)


    def test_get_elections_including_candidate_basic_data(self):
        url = '/api/v1/election/'
        filter_key = 'slug'
        filter_value = 'barbaz'
        self.data[filter_key] = filter_value
        resp = self.api_client.get(url, data=self.data)

        self.assertValidJSONResponse(resp)
        elections = self.deserialize(resp)['objects']
        self.assertEqual(type(elections[0]['candidates'][0]), dict)
        self.assertTrue(elections[0]['candidates'][0].has_key('name'))
        self.assertTrue(elections[0]['candidates'][0].has_key('id'))

    def test_get_including_unpublished_elections(self):
        self.election.published = False
        self.election.save()
        url = '/api/v1/election/'
        resp = self.api_client.get(url,data = self.data)
        self.assertValidJSONResponse(resp)
        elections = self.deserialize(resp)['objects']
        self.assertEqual(len(elections), 2) #Only my elections
        self.assertFalse(elections[0]['published'])

    def test_get_candidates_from_elections_owned_by_user(self):
        url = '/api/v1/candidate/'
        resp = self.api_client.get(url,data = self.data)
        self.assertValidJSONResponse(resp)
        candidates = self.deserialize(resp)['objects']
        self.assertEqual(len(candidates), 2)

    def test_get_candidates_per_election(self):
        url = '/api/v1/election/{0}/'.format(self.election.id)
        resp = self.api_client.get(url, data=self.data)

        self.assertValidJSONResponse(resp)
        election = self.deserialize(resp)
        candidates = election['candidates']
        self.assertEqual(len(candidates), 2)

        self.assertEqual(candidates[0]['name'], self.candidate.name)
        self.assertEqual(candidates[0]['resource_uri'], "/api/v1/candidate/{0}/".format(self.candidate.id))
        self.assertEqual(candidates[1]['name'], self.candidate2.name)
        self.assertEqual(candidates[1]['resource_uri'], "/api/v1/candidate/{0}/".format(self.candidate2.id))

    def test_an_election_shows_questions(self):
        url = '/api/v1/election/{0}/'.format(self.election.id)
        resp = self.api_client.get(url, data=self.data)
        election = self.deserialize(resp)

        self.assertTrue(election.has_key("categories"))
        self.assertEquals(len(election["categories"]), 2)

        category = election["categories"][0]

        self.assertEquals(category['name'], u"Pets and phisicians")
        self.assertTrue(category.has_key('questions'))

        self.assertEquals(len(category["questions"]), 2)
        self.assertEquals(category["questions"][0]["question"], u"¿Cuál es el nombre de la ferocidad?")
        self.assertEquals(category["questions"][1]["question"], u"¿Which one is your favourite colour?")

    def test_a_question_has_possible_answers(self):
        url = '/api/v1/election/{0}/'.format(self.election.id)
        resp = self.api_client.get(url, data=self.data)
        election = self.deserialize(resp)

        question = election["categories"][0]["questions"][0]
        self.assertTrue(question.has_key("possible_answers"))
        self.assertEquals(len(question["possible_answers"]), 2)
        self.assertEquals(question["possible_answers"][0]["caption"],u"Fiera")
        self.assertEquals(question["possible_answers"][1]["caption"],u"Ratón inteligente pero barza")

    def test_candidate_detail_shows_personal_data(self):
        url = '/api/v1/candidate/{0}/'.format(self.candidate.id)
        resp = self.api_client.get(url, data=self.data)
        candidate = self.deserialize(resp)
        self.assertTrue("personal_data" in candidate)
        self.assertFalse("resource_uri" in candidate["personal_data"][0])
        self.assertEqual(candidate["personal_data"][0]["label"], "Age")
        self.assertEqual(candidate["personal_data"][1]["label"], "Profession")
        self.assertEqual(candidate["personal_data"][0]["value"], "2")
        self.assertEqual(candidate["personal_data"][1]["value"], "Perro")

    def test_get_the_most_ferocious_twitter(self):
        url = '/api/v1/candidate/{0}/'.format(self.candidate2.id)
        resp = self.api_client.get(url, data=self.data)
        candidate = self.deserialize(resp)

        self.assertTrue("links" in candidate)
        self.assertEquals(candidate["links"].__len__(), 1)#ONLY THE FEROCIOUS TWITTER
        self.assertEquals(candidate["links"][0]["name"], self.ferocious_link.name)
        self.assertEquals(candidate["links"][0]["url"], self.ferocious_link.url)

    def test_candidate_detail_shows_personal_data_extends(self):
        background_category = BackgroundCategory.objects.create(
            name = 'categoria',
            election = self.candidate.election
            )
        backgrounds = Background.objects.create(
            name = 'background name',
            category = background_category
            )
        candidate_background = BackgroundCandidate.objects.create(
            candidate = self.candidate,
            background = backgrounds,
            value = 'candidate background value'
            )

        url = '/api/v1/candidate/{0}/'.format(self.candidate.id)
        resp = self.api_client.get(url, data=self.data)
        candidate = self.deserialize(resp)
        self.assertTrue("background" in candidate)
        self.assertEquals(candidate["background"][0]["name"],"background name")
        self.assertEquals(candidate["background"][0]["value"],"candidate background value")
